import numpy as np
import pylab as pl
import scipy as sp
from scipy import linalg


def laplace(f,N):
    h=1/(N+1)
    A=np.eye(N)-np.diag(np.ones(N-1),-1)
    A += A.T+h*h*np.eye(N)
    u=h*h*np.linalg.solve(A,f)
    return u
    

def laplaceb(f,N):
    h=1/(N+1)
    A=np.eye(N)-np.diag(np.ones(N-1),-1)
    A+=A.T
    A[N-1,N-1]=1
    A += A.T+h*h*np.eye(N)
    u=h*h*np.linalg.solve(A,f)
    return u


def laplacec(f,alpha,N):
    h=1/(N+1)
    A=np.eye(N)-np.diag(np.ones(N-1),-1)
    A+=A.T-alpha*h*A
    u=h*h*np.linalg.solve(A,f)
    return u

def diffini(n):
    h=1/(n+1)
    h2=h*h
    pi2=np.pi**2
    x,y=np.mgrid[0:(n+2)*h:h,0:(n+2)*h:h]
    ue=np.cos(np.pi*(x+y))
    d=4+2*h2*pi2*np.ones((n+2,n+2)) #((len(x),len(y)))
    fh=h2*2*pi2*np.cos(np.pi*(x+y))
    uh=np.zeros((n+2,n+2))
    aux=h*np.arange(0,n+2)
    uh[:,0]=np.cos(np.pi*aux)
    uh[:,-1]=np.cos(np.pi*(aux+1))
    uh[0,:]=np.cos(np.pi*aux)
    uh[-1,:]=np.cos(np.pi*(aux+1))
    er=np.zeros((n+2,n+2))
    v=uh.copy()
    nt=100
    for n in np.arange(0,nt):
        v[1:-1,1:-1]=(fh[1:-1,1:-1]+uh[:-2,1:-1]\
                      +uh[2:,1:-1]+uh[1:-1,:-2]\
                      +uh[1:-1,2:])/d[1:-1,1:-1]
        er=np.abs(uh-v)
        uh=v.copy()
    err=np.max(er)
    return x,y,uh,ue,err


def eqc(N,dt,T):
    pi2=np.pi**2
    h=1/(N+1)
    r=dt/h/h
    x=np.arange(0,(N+2)*h,h)
    t=np.arange(0,T+dt,dt)
    xx,tt=np.mgrid[0:(N+2)*h:h,0:T+dt:dt]
    v=np.zeros((N+2,len(t)))
    v[:,0]=np.sin(np.pi*x)
    v[0,:]=0
    A=np.eye(N)-np.diag(np.ones((1,N-1)),1)
    A+=A.T
    B=np.eye(N)+r*A
    for i in np.arange(1,len(t)):
        v[1:-1,i]=np.linalg.solve(B,v[1:-1,i-1])
    return xx,tt,v
            



        
